/* Generated automatically. */
static const char configuration_arguments[] = "/home/freetz/freetz-ng/source/toolchain-mipsel_gcc-4.6.4/gcc-4.6.4/configure --enable-option-checking --prefix=/home/freetz/freetz-ng/toolchain/build/mipsel_gcc-4.6.4/mipsel-unknown-linux-gnu --build=x86_64-pc-linux-gnu --host=x86_64-pc-linux-gnu --target=mipsel-unknown-linux-gnu --enable-languages=c --disable-shared --with-newlib --disable-libssp --with-gnu-as --with-gnu-ld --without-headers --disable-threads --disable-multilib --with-arch=mips32 --with-abi=32 --with-float=soft --disable-decimal-float --with-gmp=/home/freetz/freetz-ng/tools/build --with-mpfr=/home/freetz/freetz-ng/tools/build --with-mpc=/home/freetz/freetz-ng/tools/build --disable-nls";
static const char thread_model[] = "single";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { "abi", "32" }, { "arch", "mips32" }, { "float", "soft" }, { "llsc", "llsc" }, { "synci", "no-synci" } };
